@@ -4,6 +4,7 @@ import _nfer
 import time
 import logging
 from nfer.rules import Rule
+from nfer.normalize import normalize_interval_name, denormalize_interval_name
 
 """
 This file contains the primary API for monitoring and reporting intervals
@@ -77,12 +78,16 @@ def nfer_daemon():
                     log.debug('Running thread: %s' % thread.name)
             continue
 
+        # normalize the name before passing to nfer
+        norm_name = normalize_interval_name(interval.name)
+
         # when an interval is pulled from the queue, pass it to the C backend
-        result = _nfer.add(interval.name, interval.begin, interval.end, interval.data)
+        result = _nfer.add(norm_name, interval.begin, interval.end, interval.data)
         # then try sending any generated intervals to the callbacks
         if result is not None:
             for (name, begin, end, data) in result:
-                i = Interval(name, begin, end, data)
+                # denormalize the names that come back from nfer
+                i = Interval(denormalize_interval_name(name), begin, end, data)
                 if name in callbacks.keys():
                     for callback in callbacks[name]:
                         callback(i)
