@@ -1,6 +1,7 @@
 from uuid import uuid4
 from enum import Enum
 
+from nfer.normalize import normalize_interval_name
 import _nfer
 
 # requirements:
@@ -101,12 +102,17 @@ class Rule:
             where_str = " where {}".format(" & ".join(where))
 
         if self.operator == Operator.atomic:
-            return "{} :- {}{}{}".format(self.result, lhs, where_str, map_str)
+            interval_expr = "{} :- {}".format(self.result, lhs)
         elif self.exclusion:
-            return "{} :- {} unless {} {}{}{}".format(self.result, lhs, self.operator.name, rhs, where_str, map_str)
+            interval_expr = "{} :- {} unless {} {}".format(self.result, lhs, self.operator.name, rhs)
         else:
-            return "{} :- {} {} {}{}{}".format(self.result, lhs, self.operator.name, rhs, where_str, map_str)
+            interval_expr = "{} :- {} {} {}".format(self.result, lhs, self.operator.name, rhs)
 
+        where_map = "{}{}".format(where_str, map_str)
+
+        # normalize the interval expression but not the where or map
+        # this gets dicey
+        return normalize_interval_name(interval_expr) + where_map
 
 class AtomicRule(Rule):
     def __init__(self, lhs):
