@@ -70,17 +70,17 @@ function intervalUI() {
     var marginTopNav = svgHeight - margin.bottom - heightNav - margin.topNav;
     width = svgWidth - margin.left - margin.right - yAxisDim;
     widthNav = width;
-    modCount = Math.floor((height / yDomain.length) / intervalHeight) - 2;
+    modCount = Math.floor((height / (yDomain.length + 1)) / intervalHeight) - 1;
 
-	// hidden canvas for determining what tooltip to show
-	hiddenCanvas = selection.append('canvas')
-		.attr('width', width)
-		.attr('height', height)
-		.attr("class", "canvas-hidden")
-		.style("position", "absolute")
-        .style("left", (margin.left + yAxisDim) + "px")
-        .style("top", marginTop + "px")
-        .style("display", "none");
+    // hidden canvas for determining what tooltip to show
+    hiddenCanvas = selection.append('canvas')
+      .attr('width', width)
+      .attr('height', height)
+      .attr("class", "canvas-hidden")
+      .style("position", "absolute")
+          .style("left", (margin.left + yAxisDim) + "px")
+          .style("top", marginTop + "px")
+          .style("display", "none");
 
     hiddenContext = hiddenCanvas.node().getContext('2d');
 
@@ -261,40 +261,40 @@ function intervalUI() {
     canvas.on("mousemove", mousemove);
 
     function mousemove() {
-	  chart.draw(true);
+	    chart.draw(true);
 
-	  var mouseX = d3.event.layerX || d3.event.offsetX,
-		  mouseY = d3.event.layerY || d3.event.offsety,
-		  col = hiddenContext.getImageData(mouseX, mouseY, 1, 1).data,
+	    var mouseX = d3.event.layerX || d3.event.offsetX,
+		      mouseY = d3.event.layerY || d3.event.offsety,
+		      col = hiddenContext.getImageData(mouseX, mouseY, 1, 1).data,
           colKey = 'rgb(' + col[0] + ',' + col[1] + ',' + col[2] + ')',
-		  nodeData = colorToNode[colKey],
-		  tipData = [], i = 0, key;
+		      nodeData = colorToNode[colKey],
+		      tipData = [], i = 0, key;
 
       //console.log("layerXY: [" + d3.event.layerX + "," + d3.event.layerY + "]"+
       //            "offsetXY: [" + d3.event.offsetX + "," + d3.event.offsetY + "] " + colKey);
 
-	  if (nodeData) {
-	    // add the begin/end times to the tooltip
-	    tipData.push("begin: " + nodeData.start.getTime());
-	    tipData.push("end: " + nodeData.end.getTime());
+      if (nodeData) {
+        // add the begin/end times to the tooltip
+        tipData.push("begin: " + nodeData.start.getTime());
+        tipData.push("end: " + nodeData.end.getTime());
 
-	    if (tooltips.length > 0) {
-	      //console.log(nodeData);
-          for (; i < tooltips.length; i++) {
-            key = tooltips[i];
-            if (key in nodeData.data) {
-              tipData.push(key + ": " + nodeData.data[key]);
+        if (tooltips.length > 0) {
+          //console.log(nodeData);
+            for (; i < tooltips.length; i++) {
+              key = tooltips[i];
+              if (key in nodeData.data) {
+                tipData.push(key + ": " + nodeData.data[key]);
+              }
             }
           }
-        }
 
-		tooltip.style('opacity', 0.8)
-	      .style('top', mouseY + marginTop + 5 + 'px')
-          .style('left', mouseX + margin.left + yAxisDim + 5 + 'px')
-          .html(tipData.join("<br />"));
-	  } else {
-	    tooltip.style('opacity', 0);
-	  }
+      tooltip.style('opacity', 0.8)
+            .style('top', mouseY + marginTop + 5 + 'px')
+            .style('left', mouseX + margin.left + yAxisDim + 5 + 'px')
+            .html(tipData.join("<br />"));
+      } else {
+        tooltip.style('opacity', 0);
+      }
     }
 
     return chart;
@@ -304,16 +304,16 @@ function intervalUI() {
   var colorToNode = {}, nextCol = 1;
   // function to create new colors for picking
   var genColor = function() {
-	var ret = [], color;
-	if (nextCol < 16777215){
-	  ret.push(nextCol & 0xff);             // R
-	  ret.push((nextCol & 0xff00) >> 8);    // G
-	  ret.push((nextCol & 0xff0000) >> 16); // B
-      // more separation to avoid anti-alias related mismatches
-	  nextCol += 3;
-    }
-	color = "rgb(" + ret.join(',') + ")";
-	return color;
+    var ret = [], color;
+    if (nextCol < 16777215){
+      ret.push(nextCol & 0xff);             // R
+      ret.push((nextCol & 0xff00) >> 8);    // G
+      ret.push((nextCol & 0xff0000) >> 16); // B
+        // more separation to avoid anti-alias related mismatches
+      nextCol += 3;
+      }
+    color = "rgb(" + ret.join(',') + ")";
+    return color;
   }
 
   chart.draw = function(isHidden) {
@@ -324,6 +324,11 @@ function intervalUI() {
     if (!isHidden) {
       context.fillStyle = "#F5F5F5";
       context.fillRect(0, 0, width, height);
+      // draw some alternating colors for better visual distinction
+      context.fillStyle = "#E5E5E5";
+      for (var i = 0; i < yDomain.length; i += 2) {
+        context.fillRect(0, (height / (yDomain.length + 1)) * i, width, (height / (yDomain.length + 1)));
+      }
       navContext.clearRect(0, 0, widthNav, heightNav);
       navContext.fillStyle = "#F5F5F5";
       navContext.fillRect(0, 0, widthNav, heightNav);
@@ -333,13 +338,13 @@ function intervalUI() {
     for (var i = 0; i < data.length; i++) {
       var d = data[i],
           dx = Math.round(x(d.start)),
-          dy = Math.round(y(d.name) - (intervalHeight / 2) + ((d.count % modCount) * intervalHeight)),
+          dy = Math.round(y(d.name) + (intervalHeight * d.vertOffset)),
           dw = Math.round(x(d.end) - x(d.start)),
           nx = xNav(d.start),
           ny = yNav(d.name) - 0.5,
           nw = xNav(d.end) - xNav(d.start),
           cx = Math.round(x(d.end)),
-          cy = Math.round(y(d.name) + ((d.count % modCount) * intervalHeight)),
+          cy = Math.round(y(d.name) + (intervalHeight * d.vertOffset)),
           ncx = xNav(d.end),
           ncy = yNav(d.name);
 
@@ -348,31 +353,33 @@ function intervalUI() {
 
       } else {
         // apply the color
-        context.fillStyle = colors(d.data.color || 0);
+        var fillColor = (d.name in colorPerY ? colorPerY[d.name] : 0);
+        context.fillStyle = colors(d.data.color || fillColor);
         // for the nav, just use black
         navContext.fillStyle = "black";
       }
 
       if (d.start.getTime() != d.end.getTime()) {
         // draw a rectangle
-        c.fillRect(dx, dy, dw, intervalHeight);
+        c.fillRect(dx, dy, dw, intervalHeight - 1);
         if (!isHidden) {
           // draw in the nav
           navContext.fillRect(nx, ny, nw, 1);
         }
 
       } else {
-         c.beginPath();
-         c.arc(cx, cy, intervalHeight / 2, 0,  2 * Math.PI, true);
-         c.fill();
-         c.closePath();
+        //draw a circle if it's zero duration
+        c.beginPath();
+        c.arc(cx, cy, intervalHeight / 2, 0,  2 * Math.PI, true);
+        c.fill();
+        c.closePath();
 
-         if (!isHidden) {
-           navContext.beginPath();
-           navContext.arc(ncx, ncy, 1, 0,  2 * Math.PI, true);
-           navContext.fill();
-           navContext.closePath();
-         }
+        if (!isHidden) {
+          navContext.beginPath();
+          navContext.arc(ncx, ncy, 1, 0,  2 * Math.PI, true);
+          navContext.fill();
+          navContext.closePath();
+        }
       }
     }
   }
@@ -438,12 +445,42 @@ function intervalUI() {
   // chart accessors and methods
   // TODO: change this because this is JS, not Python or Java and this isn't even really a class
 
+  // track the max timestamp of the verticle offsets for each interval name
+  var maxTsPerYVO = {};
+  var calculateVerticalOffset = function(toAdd) {
+    // if the name hasn't appeared yet, set it and return
+    if (!(toAdd.name in maxTsPerYVO)) {
+      maxTsPerYVO[toAdd.name] = [];
+      maxTsPerYVO[toAdd.name][0] = toAdd.end;
+      return 0;
+    }
+
+    // figure out if there needs to be a vertical offset
+    for (var j = 0; j < modCount; j++) {
+      // if anything has been set at this vertical offset, we must check it's max ts
+      // if nothing has been set at it or the max ts is lower, then we are good to use it
+      if (j >= maxTsPerYVO[toAdd.name].length ||
+          maxTsPerYVO[toAdd.name][j] < toAdd.start) {
+        // we can use this vertical offset
+        // store the max ts
+        maxTsPerYVO[toAdd.name][j] = toAdd.end;
+        // return
+        return j;
+      }
+    }
+    // if no free vertical offset is found, we will just use zero
+    // not ideal but it will be okay most of the time
+    return 0;
+  }
+
   // new interval. This most recent interval will appear
   // on the far right side of the chart
   chart.add = function(toAdd) {
     // for vertical positioning
     toAdd.count = counts[toAdd.name];
     counts[toAdd.name]++;
+    toAdd.vertOffset = calculateVerticalOffset(toAdd);
+
     // for tooltip tracking
     toAdd.tracking = genColor();
     colorToNode[toAdd.tracking] = toAdd;
@@ -468,6 +505,7 @@ function intervalUI() {
       // for vertical positioning
       toAdd[i].count = counts[toAdd[i].name] || 0;
       counts[toAdd[i].name] = toAdd[i].count + 1;
+      toAdd[i].vertOffset = calculateVerticalOffset(toAdd[i]);
       // for tooltip tracking
       toAdd[i].tracking = genColor();
       colorToNode[toAdd[i].tracking] = toAdd[i];
@@ -529,15 +567,25 @@ function intervalUI() {
     return chart;
   }
 
+  // track default interval colors
+  // these are default color-blind accessible, high-contrast colors
+  // var DEFAULT_COLORS = ['#4477AA', '#EE6677', '#228833', '#CCBB44', '#66CCEE', '#AA3377', '#BBBBBB'];
+  var nextColor = 0;
+  var colorPerY = {};
   // yItems (can be dynamically added after chart construction)
   chart.yDomain = function(_) {
     if (arguments.length == 0) return yDomain;
     yDomain = _;
-    // update the hash of item counts
+    // update the hash of item counts and the max timestamps
     for (var i = 0; i < _.length; i++) {
       if (!(_[i] in counts)) {
         counts[_[i]] = 0;
       }
+
+      // update the colors (try to preserve them)
+      if (!(_[i] in colorPerY)) {
+        colorPerY[_[i]] = nextColor++;
+      }
     }
     if (svg) {
       // update the y ordinal scale
@@ -547,11 +595,17 @@ function intervalUI() {
       // update the y ordinal scale for the nav chart
       yNav = d3.scalePoint().domain(yDomain).rangeRound([heightNav, 0]).padding([1]);
       // update the mod number for count offset
-      modCount = Math.floor((height / _.length) / intervalHeight);
+      modCount = Math.floor((height / (_.length + 1)) / intervalHeight) - 1;
     }
+
     return chart;
   }
 
+  // expose the max ts var
+  chart.getYVOs = function() {
+    return maxTsPerYVO;
+  }
+
   // set the maximum number of seconds displayed in the chart
   chart.maxSeconds = function(_) {
     if (arguments.length == 0) return maxSeconds;
