@@ -22,7 +22,7 @@
  *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-
+#include <assert.h>
 #include <stdlib.h>
 #include <stdio.h>
 
@@ -474,7 +474,7 @@ static nfer_rule * generate_each_rule(ast_node *node, nfer_specification *spec,
     nfer_rule *rule;
     map_iterator mit;
     map_key key;
-    map_value value, expression;
+    map_value value, expression, equivalence;
     int size;
 
     if (!node) {
@@ -488,6 +488,14 @@ static nfer_rule * generate_each_rule(ast_node *node, nfer_specification *spec,
                 rule = add_rule_to_specification(spec, result, node->atomic_interval_expr.result_id, ALSO_OPERATOR, WORD_NOT_FOUND, NULL);
                 // always hidden!
                 rule->hidden = true;
+                // separate is set when we remap, meaning the two labels are equivalent
+                // capture this by setting a key in the equivalent_labels map
+                // this is later used to check interval equivalence
+                equivalence.type = string_type;
+                equivalence.value.string = node->atomic_interval_expr.result_id;
+                map_set(&spec->equivalent_labels, result, &equivalence);
+                filter_log_msg(LOG_LEVEL_DEBUG, "    Remapping %u to %u for seprate AIE rule\n", result, node->atomic_interval_expr.result_id);
+                // initialize the rule map expressions
                 initialize_map(&rule->map_expressions);
 
                 if (where_node) {
@@ -583,86 +591,81 @@ static void generate_eval_from_map_expr_list(ast_node *ie_node, ast_node *map_ex
     generate_eval_from_map_expr_list(ie_node, map_expr_list->map_expr_list.tail, map);
 }
 
-bool in_imports(ast_node *imports, word_id name) {
-    if (!imports) {
-        return false;
-    }
-    if (imports->import_list.import == name) {
-        return true;
-    } else {
-        return in_imports(imports->import_list.tail, name);
-    }
-}
-
-static void generate_rules(ast_node *node, nfer_specification *spec, bool first, ast_node *imports) {
+static void generate_rules(ast_node *node, nfer_specification *spec) {
     nfer_rule *rule;
     int size;
 
+    /* preconditions : not null and node should be a rule */
+    assert(node != NULL);
+    assert(spec != NULL);
+    assert(node->type == type_rule);
+
+    // skip generation if the rule is disabled
+    if (node->rule.disabled) {
+        return;
+    }
+
+    rule = generate_each_rule(node->rule.interval_expr, spec, node->rule.result_id, node->rule.where_expr);
+    if (!rule) {
+        // the rule is atomic, we need to generate it here
+        rule = add_rule_to_specification(spec, node->rule.result_id, node->rule.interval_expr->atomic_interval_expr.result_id,
+                    ALSO_OPERATOR, WORD_NOT_FOUND, NULL);
+
+        if (node->rule.where_expr) {
+            size = 1 + get_eval_size(node->rule.interval_expr, node->rule.where_expr);
+            initialize_expression_input(&rule->where_expression, size);
+            rule->where_expression[0].length = size;
+            filter_log_msg(LOG_LEVEL_DEBUG, "    Generating eval for atomic rule where clause, eval size is %d\n", size);
+            generate_eval_from_expr(node->rule.interval_expr, node->rule.where_expr, rule->where_expression, 1);
+        }
+    }
+
+    // make sure this is not a hidden rule
+    rule->hidden = false;
+    if (node->rule.end_points) {
+        size = 1 + get_eval_size(node->rule.interval_expr, node->rule.end_points->end_points.begin_expr);
+        initialize_expression_input(&rule->begin_expression, size);
+        rule->begin_expression[0].length = size;
+        generate_eval_from_expr(node->rule.interval_expr, node->rule.end_points->end_points.begin_expr, rule->begin_expression, 1);
+
+        size = 1 + get_eval_size(node->rule.interval_expr, node->rule.end_points->end_points.end_expr);
+        initialize_expression_input(&rule->end_expression, size);
+        rule->end_expression[0].length = size;
+        generate_eval_from_expr(node->rule.interval_expr, node->rule.end_points->end_points.end_expr, rule->end_expression, 1);
+    }
+    if (node->rule.map_expr_list) {
+        generate_eval_from_map_expr_list(node->rule.interval_expr, node->rule.map_expr_list, &rule->map_expressions);
+    }
+}
+
+/**
+ * Generate the nfer_specification for a DSL AST after it has been processed
+ * by semantic analysis.
+ * This function recurses over the modules and rule lists and calls
+ * generate_rules for each rule in turn.
+ **/
+void generate_specification(ast_node *node, nfer_specification *spec) {
     if (!node) {
         return;
     }
     switch (node->type) {
-    case type_rule:
-        rule = generate_each_rule(node->rule.interval_expr, spec, node->rule.result_id, node->rule.where_expr);
-        if (!rule) {
-            // the rule is atomic, we need to generate it here
-            rule = add_rule_to_specification(spec, node->rule.result_id, node->rule.interval_expr->atomic_interval_expr.result_id,
-                        ALSO_OPERATOR, WORD_NOT_FOUND, NULL);
-
-            if (node->rule.where_expr) {
-                size = 1 + get_eval_size(node->rule.interval_expr, node->rule.where_expr);
-                initialize_expression_input(&rule->where_expression, size);
-                rule->where_expression[0].length = size;
-                filter_log_msg(LOG_LEVEL_DEBUG, "    Generating eval for atomic rule where clause, eval size is %d\n", size);
-                generate_eval_from_expr(node->rule.interval_expr, node->rule.where_expr, rule->where_expression, 1);
-            }
-        }
-
-        // make sure this is not a hidden rule
-        rule->hidden = false;
-        if (node->rule.end_points) {
-            size = 1 + get_eval_size(node->rule.interval_expr, node->rule.end_points->end_points.begin_expr);
-            initialize_expression_input(&rule->begin_expression, size);
-            rule->begin_expression[0].length = size;
-            generate_eval_from_expr(node->rule.interval_expr, node->rule.end_points->end_points.begin_expr, rule->begin_expression, 1);
-
-            size = 1 + get_eval_size(node->rule.interval_expr, node->rule.end_points->end_points.end_expr);
-            initialize_expression_input(&rule->end_expression, size);
-            rule->end_expression[0].length = size;
-            generate_eval_from_expr(node->rule.interval_expr, node->rule.end_points->end_points.end_expr, rule->end_expression, 1);
-        }
-        if (node->rule.map_expr_list) {
-            generate_eval_from_map_expr_list(node->rule.interval_expr, node->rule.map_expr_list, &rule->map_expressions);
-        }
-
-
-        break;
     case type_rule_list:
-        generate_rules(node->rule_list.head, spec, false, imports);
-        generate_rules(node->rule_list.tail, spec, false, imports);
+        // in the case of a rule_list, call generate_rules on the head (a rule node)
+        generate_rules(node->rule_list.head, spec);
+        // then recurse on the rest of the rule_list
+        generate_specification(node->rule_list.tail, spec);
         break;
     case type_module_list:
-
-        if (first) {
-            generate_rules(node->module_list.rules, spec, true, node->module_list.imports);
-            generate_rules(node->module_list.tail, spec, false, node->module_list.imports);
-        } else {
-            if (in_imports(imports, node->module_list.id)) {
-                generate_rules(node->module_list.rules, spec, false, node->module_list.imports);
-            } else {
-                filter_log_msg(LOG_LEVEL_DEBUG, "Ignoring non-imported module %d\n", node->module_list.id);
-            }
-            generate_rules(node->module_list.tail, spec, false, imports);
-        }
-
+        // only process rules from imported modules
+        if (node->module_list.imported) {
+            generate_specification(node->module_list.rules, spec);
+        } 
+        // recurse on the rest of the module list
+        generate_specification(node->module_list.tail, spec);
         break;
     default:
         return;
     }
 }
 
-void generate_specification(ast_node *node, nfer_specification *spec) {
-    generate_rules(node, spec, true, NULL);
-}
-
 
